<?php

namespace Drupal\usermerge_og\Plugin\UserMerge\Property;

use Drupal\Core\Entity\EntityInterface;
use Drupal\og\Og;
use Drupal\og\OgMembershipInterface;
use Drupal\user\UserInterface;
use Drupal\usermerge\Plugin\UserMerge\Property\UserMergePropertyBase;

/**
 * Og membership property.
 *
 * @UserMergeProperty(
 *   id = "property_og_membership",
 *   name = @Translation("Og memberships"),
 *   description = @Translation("Move user memberships from retired to retained users."),
 *   review = "",
 * )
 * @package Drupal\usermerge\Plugin\UserMerge\Property
 */
class PropertyOgMembership extends UserMergePropertyBase {

  /**
   * {@inheritDoc}
   */
  public function process(UserInterface $retired, UserInterface $retained, array $settings = []): void {
    $retired_user_memberships = Og::getMemberships($retired);

    // Loop through memberships.
    foreach ($retired_user_memberships as $membership) {
      // Get the group from membership.
      $group = $membership->getGroup();
      if (empty($group)) {
        continue;
      }

      if ($this->membershipAlreadyExists($retained, $group)) {
        // Do nothing if retained user already has a membership to the same
        // group.
        continue;
      }

      $membership->setOwner($retained);
      $membership->save();
    }
  }

  /**
   * Check if the membership already exist for the user for the group.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user.
   * @param \Drupal\Core\Entity\EntityInterface $group
   *   The group.
   *
   * @return bool
   *   TRUE, if the membership already exist for the user for the group.
   */
  public function membershipAlreadyExists(UserInterface $user, EntityInterface $group) : bool {
    $membership = Og::getMembership($group, $user, OgMembershipInterface::ALL_STATES);

    return !empty($membership);
  }

}
