<?php

namespace Drupal\usermerge_og;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\og\Og;
use Drupal\user\UserInterface;

/**
 * Alter the usermerge review form.
 */
class ReviewUserFormAlter {

  use StringTranslationTrait;

  /**
   * Form alter for the user review form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function formAlter(array &$form, FormStateInterface $form_state) {
    $multi_step_storage = $form_state->getFormObject()->getMultiStepStorage();
    $retire = $multi_step_storage->getRetiredAccount();
    $retain = $multi_step_storage->getRetainedAccount();

    // Create a new table to display the entities owned by both users for
    // side-by-side comparison.
    $form['data']['og_memberships'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Group memberships for retired user'),
        $this->t('Group memberships for retained user'),
      ],
    ];

    $retire_memberships_info = $this->buildMembershipsInfo($retire);
    $retain_memberships_info = $this->buildMembershipsInfo($retain);

    // Add both data to table.
    $form['data']['og_memberships'][] = [
      'retire' => $retire_memberships_info,
      'retain' => $retain_memberships_info,
    ];
  }

  /**
   * Build membership info for user.
   *
   * This will be used to review and compare user memberships in the user review
   * form.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user.
   *
   * @return array
   *   Render array for user membership information.
   */
  protected function buildMembershipsInfo(UserInterface $user) {
    $memberships_info = [];
    $memberships = Og::getMemberships($user);
    foreach ($memberships as $membership) {
      $group = $membership->getGroup();
      $link = Link::fromTextAndUrl($group->label(), $group->toUrl())->toString();
      $memberships_info[] = $this->t('@entity_type : @bundle : @entity_link : @state', [
        '@entity_type' => $group->getEntityType()->getLabel(),
        '@bundle' => \Drupal::entityTypeManager()
          ->getStorage($group->getEntityType()->getBundleEntityType())
          ->load($group->bundle())
          ->label(),
        '@entity_link' => $link,
        '@state' => $membership->getState(),
      ]);
    }

    $memberships_info = [
      '#theme' => 'item_list',
      '#items' => $memberships_info,
    ];

    return $memberships_info;
  }

}
